class InvalidState(Exception):
    ...

class DriverState(InvalidState):
    ...


class SensorException(Exception):
    ...


class SensorNotFound(SensorException):
    ...


class InconsistentConfiguration(SensorException):
    ...


class IncompleteBufferError(SensorException):
    ...


class ResourceError(SensorException):
    ...


class NoFreeHardwareTrigger(ResourceError):
    @staticmethod
    def create():
        return NoFreeHardwareTrigger('No free hardware trigger available.')


class NoFreeSoftwareTrigger(ResourceError):
    @staticmethod
    def create():
        return NoFreeSoftwareTrigger('No free software trigger available.')


class NoFreeRois(ResourceError):
    @staticmethod
    def create():
        return NoFreeRois('No free regions of interest available.')


#
# Errors that convey, that a specific feature (currently) does not work
#

class FeatureUnavailable(Exception):
    """The feature does not currently"""


class FeatureImplementationError(FeatureUnavailable):
    """The feature is available, but in the current configuration an error occurred."""


class FeatureNotImplemented(FeatureUnavailable):
    """The driver did not implement the feature."""


class SensorFeatureNotAvailable(SensorException, FeatureUnavailable):
    """The sensor does not provide / implement the feature."""


class SensorFeatureCurrentlyNotAvailable(SensorFeatureNotAvailable, FeatureUnavailable):
    """The sensor does not support this feature in its current state."""
